package com.scbfkj.uni.api;

import com.scbfkj.uni.library.UniReturnUtil;
import jakarta.servlet.http.HttpServletRequest;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.Map;

@Component
@Aspect
public class LogAop {

    @Pointcut("within( com.scbfkj.uni.api.*)")
    public void pointcut() {

    }

    @Around(value = "pointcut() && args(headers,body)", argNames = "joinPoint,headers,body")
    public Object invoke(ProceedingJoinPoint joinPoint, Map<String, Object> headers, Map<String, Object> body) {
//        当前时间戳
        long beforeTime = System.currentTimeMillis();
//        请求
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.currentRequestAttributes();
//
        HttpServletRequest request = requestAttributes.getRequest();
        String uri = request.getRequestURI();
        String ip = getIpAddr(request);
//        请求参数
        Object[] args = joinPoint.getArgs();
//        返回结果
        Object proceed = null;
        try {
            proceed = joinPoint.proceed(args);
        } catch (Throwable e) {
//            错误异常消息
            proceed = UniReturnUtil.fail(e.getMessage());
        }
//        执行后时间戳
        long afterTime = System.currentTimeMillis();
//        执行时间
        long execTime = afterTime - beforeTime;
        System.out.println("执行时长："+execTime);
//        返回结果
        return proceed;
    }

    public String getIpAddr(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }
}
