package com.scbfkj.uni.library;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.fasterxml.jackson.datatype.jdk8.Jdk8Module;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import jakarta.json.JsonObject;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.Temporal;

public final class DataFormatUtil {
    private final static ObjectMapper objectMapper = new ObjectMapper();

    static {
        objectMapper.registerModule(new Jdk8Module());
        objectMapper.registerModule(new JavaTimeModule());
        objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
    }

    public static JsonNode stringToJsonNode(String source) throws JsonProcessingException {
        return objectMapper.readTree(source);
    }

    public static ObjectNode stringToObjectNode(String source) throws JsonProcessingException {
        return objectMapper.readValue(source, ObjectNode.class);
    }

    public static ArrayNode stringToArrayNode(String source) throws JsonProcessingException {
        return objectMapper.readValue(source, ArrayNode.class);
    }

    public static String toString(Object source) throws JsonProcessingException {
        if (source instanceof String || source instanceof Number)
            return source.toString();
        if (source instanceof LocalDate result) result.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        if (source instanceof LocalDateTime result) result.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        return objectMapper.writeValueAsString(source);
    }

    public static LocalDate stringToDate(String dateStr) {
        return LocalDate.parse(dateStr);
    }

    public static LocalDateTime stringToDateTime(String dateTimeStr) {
        return LocalDateTime.parse(dateTimeStr);
    }
}
