package com.scbfkj.uni.process;

import com.scbfkj.uni.library.DbUtil;
import com.scbfkj.uni.system.Config;
import com.zaxxer.hikari.pool.HikariPool;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

public class DataBase {

    public static List<Map<String, Object>> query(String connectionStr, String sql, List<Object[]> argsList) throws Exception {
        HikariPool dataSourcePool = Config.getDataSourcePool(connectionStr);

        List<Map<String, Object>> result = new ArrayList<>();
        for (Object[] args : argsList) {
            try (Connection connection = dataSourcePool.getConnection();
                 PreparedStatement preparedStatement = connection.prepareStatement(sql)
            ) {
                for (int i = 0; i < args.length; i++) {
                    preparedStatement.setObject(i + 1, args[i]);
                }
                ResultSet resultSet = preparedStatement.executeQuery();
                result.addAll(DbUtil.getResult(connectionStr, sql, resultSet));
            }

        }

        return result;

    }

    public static int[] updateBatch(String connectionStr, String sql, List<Object[]> argsList) throws Exception {
        HikariPool dataSourcePool = Config.getDataSourcePool(connectionStr);
        try (Connection connection = dataSourcePool.getConnection();
             PreparedStatement preparedStatement = connection.prepareStatement(sql)
        ) {
            for (Object[] args : argsList) {
                for (int i = 0; i < args.length; i++) {
                    preparedStatement.setObject(i + 1, args[i]);
                }
                preparedStatement.addBatch();
                try {
                    connection.commit();
                } catch (Exception e) {
                    connection.rollback();
                    throw e;
                }
            }
            return preparedStatement.executeBatch();
        }
    }

    public static int exec(String connectionStr, String sql) throws Exception {
        HikariPool dataSourcePool = Config.getDataSourcePool(connectionStr);
        try (Connection connection = dataSourcePool.getConnection();
             PreparedStatement preparedStatement = connection.prepareStatement(sql)
        ) {
            int i = preparedStatement.executeUpdate();
            try {
                connection.commit();
            } catch (Exception e) {
                connection.rollback();
                throw e;
            }
            return i;
        }
    }

    public static List<Map<String, Object>> query(String connectionStr, String sql, List<Map<String, Object>> argsList, List<String> filterColumns, Map<String, Object> filterLines) throws Exception {

        sql = sql.replaceAll("(\\r)?\\n", " ");

        if (Objects.nonNull(filterLines) && !filterLines.isEmpty()) {
            sql = " (%s %s %s) ".formatted(sql, sql.contains(" where ") ? " " : " where ", filterLines.entrySet().stream().map(it -> "%s = %s".formatted(it.getKey(), it.getValue())).collect(Collectors.joining(" and ")));
        }
        if (Objects.nonNull(filterColumns) && !filterColumns.isEmpty()) {
            sql = "select %s from %s".formatted(filterColumns.stream().collect(Collectors.joining(",")), sql);
        } else {
            sql = "select * from %s".formatted(sql);
        }

        List<List<Object>> result = new ArrayList<>();
        List<String> names = new ArrayList<>();
        if (sql.matches("《.*》")) {
            Pattern compile = Pattern.compile("(?=《).*(?=》)");
            Matcher matcher = compile.matcher(sql);
            int index = 0;
            while (matcher.find()) {
                String name = matcher.group(index);
                names.add(name);
            }
        }
        if (names.size() == 1 && names.contains("whereStr")) {
            List<Map<String, Object>> list = new ArrayList<>();
            for (Map<String, Object> stringObjectMap : argsList) {
                List<Object> args = new ArrayList<>();
                StringJoiner joiner = new StringJoiner(" and ");
                for (Map.Entry<String, Object> entry : stringObjectMap.entrySet()) {
                    String formatted = "%s = ?".formatted(entry.getKey());
                    args.add(entry.getValue());
                    joiner.add(formatted);
                }
                String whereStr = " " + joiner;
                String sqlStr = sql.replace("《whereStr》", whereStr);
                List<Map<String, Object>> apply = query(connectionStr, sqlStr, new ArrayList<>() {{
                    add(args.toArray());
                }});
                list.addAll(apply);
            }
            return list;
        } else {
            for (String it : names) {
                sql = sql.replace("《%s》".formatted(it), " ? ");
            }
            for (Map<String, Object> map : argsList) {
                List<Object> args = new ArrayList<>();
                for (String name : names) {
                    args.add(map.get(name));
                }
                result.add(args);
            }
            return query(connectionStr, sql, result.stream().map(List::toArray).toList());
        }
    }

}
