package com.scbfkj.uni.system;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.oracle.truffle.api.Option;
import com.scbfkj.uni.library.DataEncryptionUtil;
import com.scbfkj.uni.library.DataFormatUtil;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.pool.HikariPool;
import jakarta.annotation.Nonnull;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;

@Component
public class Config {

    public static String centerConnectionStr;
    public static String securityConnectionStr;


    private final static String TYPE = "RSA";

    private final static String ALGORITHM = "RSA/ECB/PKCS1Padding";

    @Value("${cer.private-key}")
    private static String privateKeyStr;

    private final static Map<String, HikariPool> dataSourcePools = new HashMap<>();


    public static HikariPool getDataSourcePool(String connectionStr) throws Exception {
        if (dataSourcePools.containsKey(connectionStr)) {
            return dataSourcePools.get(connectionStr);
        }
        JsonNode jsonNode = DataFormatUtil.stringToJsonNode(connectionStr);

        JsonNode jdbcUrl = jsonNode.get("jdbcUrl");
        JsonNode username = jsonNode.get("username");
        JsonNode password = jsonNode.get("password");
        JsonNode driverClassName = jsonNode.get("driverClassName");
        HikariPool dataSourcePool = createDataSourcePool(jdbcUrl.asText(), username.asText(), password.asText(), driverClassName.asText(), "center");
        dataSourcePools.put(connectionStr, dataSourcePool);
        return dataSourcePool;
    }



    public static HikariPool createDataSourcePool(@Nonnull String url, String username, String password, String driver, String poolName) throws Exception {
        HikariConfig hikariConfig = new HikariConfig();
        hikariConfig.setJdbcUrl(DataEncryptionUtil.decryptByPrivateKey(url, privateKeyStr, TYPE, ALGORITHM));

        if (Objects.nonNull(username)) {
            hikariConfig.setUsername(DataEncryptionUtil.decryptByPrivateKey(username, privateKeyStr, TYPE, ALGORITHM));
        }
        if (Objects.nonNull(password)) {
            hikariConfig.setPassword(DataEncryptionUtil.decryptByPrivateKey(password, privateKeyStr, TYPE, ALGORITHM));
        }
        if (Objects.nonNull(driver)) {
            hikariConfig.setDriverClassName(driver);
        }
        if (Objects.nonNull(poolName)) {
            hikariConfig.setPoolName(poolName);
        }
        hikariConfig.setAutoCommit(false);
        return new HikariPool(hikariConfig);
    }
}
