package com.scbfkj.uni.system;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.node.JsonNodeFactory;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.scbfkj.uni.library.DataEncryptionUtil;
import com.scbfkj.uni.library.DataFormatUtil;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@Component
public class SystemInit {


    @Value("${db.center.url:jdbc:h2:file:./center.h2}")
    private String centerUrl;
    @Value("${db.center.username}")
    private String centerUsername = null;
    @Value("${db.center.password}")
    private String centerPassword = null;
    @Value("${db.center.driver:org.h2.Driver}")
    private String centerDriver;

    @Value("${db.security.url:jdbc:h2:file:./security.h2}")
    private String securityUrl;
    @Value("${db.security.username}")
    private String securityUsername = null;
    @Value("${db.security.password}")
    private String securityPassword = null;
    @Value("${db.security.driver:org.h2.Driver}")
    private String securityDriver;


    @PostConstruct
    public void init() throws JsonProcessingException {
        Map<String, Object> centerConnectionMap = initConnection(centerUrl, centerUsername, centerPassword, centerDriver);
        Config.centerConnectionStr = DataFormatUtil.toString(centerConnectionMap);
        Map<String, Object> securityConnectionMap = initConnection(securityUrl, securityUsername, securityPassword, securityDriver);
        Config.securityConnectionStr = DataFormatUtil.toString(securityConnectionMap);

    }

    private Map<String, Object> initConnection(String url, String username, String password, String driver) {
        HashMap<String, Object> connectionMap = new HashMap<>();
        if (Objects.nonNull(url)) {
            connectionMap.put("jdbcUrl", url);
        }
        if (Objects.nonNull(username) && StringUtils.hasText(username)) {
            connectionMap.put("username", username);
        }
        if (Objects.nonNull(password) && StringUtils.hasText(password)) {
            connectionMap.put("password", password);
        }
        if (Objects.nonNull(driver) && StringUtils.hasText(driver)) {
            connectionMap.put("driverClassName", driver);
        }
        return connectionMap;
    }
}
